%% Premièrement importer les données :
% Ici, un fichier .dat

% Exemple avec les données de plomb [id , X, Y, concentration]
load plomb.dat
data=plomb(:,2:4);

%% On appel statelem pour calculer des statistiques usuelles. 
% Tu peux modifier les graphiques de statelem en fonction de tes données
% statelem fait appel a deux fonction plotsc.m et plot2.m pour ajouter des
% fonctionnité dans le code.
statelem

%% On calcul les variogrammes d'incicatrice pour chaque seuil.
% Le programme appel varioexp2d.m pour calculer les variogrammes expérimentaux;
% Le programme appel ajustermulti.m pour ajuster un modèle anisotrope théorique;
% Bien suivre l'exemple pour l'appliquer.
% Ex. pour un seuil
seuils=[50 100 150 200 300 500 700];
ms=[25 75 125 175 250 400 600 1735]; % valeurs représentatives de chaque classe
seuil = 700; % Seuil de 250 ppm
ns = length(seuils); % Nombre de seuils;
data_seuil = [ data(:,1:2) data(:,3) < seuil ];
% Paramètres pour les variogrammes expérimentaux
nbclas=10;                           % Nombre de classe pour h
lclas=[(0:nbclas-1);(1:nbclas)]'*500;  % Discrétisation des classe
vdir=[0,1,2,3,4,5,6,7]'*22.5;        % Direction anisotropie
vreg=ones(8,1)*10;                   % Tolérance angulaire

% Les variogrammes expérimentaux
[gexp]=varioexp2d(data_seuil,nbclas,lclas,vdir,vreg);

% Paramètres pour l'ajustement des variogrammes expérimentaux
% On ajuste les variogrammes en pondérant en fonction du nombre de paire de points. 
expoh=2;                            % Ici, l'importance est exposant 2
hmin=1;                             % On ajuste par la classe h=0; gérer le bruit
ccode=[0;0];                        % (0 libre et 1 imposé)
modelcode=[1 1 1 1  ;  1 0 0 0];    % (0 libre et 1 imposé), donc on impose l'effet de pépite et le modèle exponentiel
var_seuil = var(data_seuil(:,3));
c=[0.1*var_seuil; 0.9*var_seuil];    % Valeur à changer au besoin
model=[1 1 1 1  ; 4 4000 4000 50];     % Valeur à changer au besoin
% Ajustement des variogrammes expérimentaux
[model,c,~]=ajustermulti(gexp,model,c,modelcode,ccode,vdir,vreg,expoh,hmin);

%% Maintenant les modèles ajustés est dans model et c. Il reste à kriger. 
% La grille de krigeage
xmin = 750; xmax = 11250; dx = 250;
ymin = 750; ymax = 11500; dy = 250;
x0=grille2(xmin,xmax,dx,ymin,ymax,dy);
% icode indique le type de krigeage
itype = 3; % Krigeage ordinaire.
% Moyenne des données (non utile, car krigrage ordinaire). On fournit 0.
avg = 0 ;
% Taille de block à kriger
block = [100 100];
% Discrétisation du bloc;
nd = [4 4];
% Aucune validation croisée
ival = 0;
% Nombre de données dans le voisinnage
nk = 16;
% Rayon de recherche circulaire maximal
r = 9999;
% Nombre de krigé simultannément
ntok = 1; % Un point à la fois
[krig,skrig,svbloc]=cokri(data_seuil,x0,model,c,itype,avg,block,nd,ival,nk,r,ntok);

%% Visualisation du seuil krigé
nx=length(xmin:dx:xmax);
ny=length(ymin:dy:ymax);
X=reshape(x0(:,1),nx,ny);
Y=reshape(x0(:,2),nx,ny);

figure(33);clf
t=reshape(krig(:,3),nx,ny);
contourf(X,Y,t);
title('Pb krigé (Seuil = 250) ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 1])
set(t,'fontsize',8);
t=colorbar;
%%
[krig,skrig,svbloc] = KI_auto(data, seuils);

%% On doit réaliser la correction d'ordre
% correction
[n,m]=size(krig);

fz=krig(:,3,1:ns);
fz=reshape(fz,n,ns);

fzc=corrige(fz);

%% Calcul l'espérance conditionnelle
esp=affine(fzc,1,1,ms,seuils);

figure(31);clf
contourf(X,Y,reshape(esp,nx,ny));
title('Espérance conditionnelle ');
axis('equal')
set(gca,'fontsize',8);
%caxis([0 1500]);
t=colorbar;
caxis([0 2500]);
set(t,'fontsize',8);
t=colorbar;

%% Calcul de l'écart-type conditionnel
figure(32);clf
t=fzc;
vcond=fzc(:,1)*ms(1)^2+(1-fzc(:,end))*ms(end)^2;
vcond=vcond+sum((fzc(:,2:end)-fzc(:,1:end-1)).*(ones(nx*ny,1)*ms(2:end-1).^2),2)-esp.^2;
vcond=reshape(vcond,nx,ny);
contourf(X,Y,sqrt(vcond),[0:100:1000]);
hold on;plot(data(:,1),data(:,2),'+w','markersize',2);hold off;
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 700])
t=colorbar;
set(t,'fontsize',8);
title('Écart-type conditionnel ');

%% Calcul du krigeage
% On réalise le krigeage automatiquement
[krig,skrig,svbloc] = K_auto(data);

figure(33);clf
t=reshape(krig(:,3,1),nx,ny);
contourf(X,Y,t);
title('Pb krigé ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 2500])
set(t,'fontsize',8);
t=colorbar;

figure(34);clf
t=reshape(skrig(:,3,1),nx,ny);
contourf(X,Y,sqrt(t),[0:100:1000]);
hold on;plot(data(:,1),data(:,2),'+w','markersize',2);hold off;
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 700])
t=colorbar;
set(t,'fontsize',8);
title('Écart-type de krigeage')
%% D'autres cartes
figure(35);clf
t=1-reshape(fzc(:,6),nx,ny);
contourf(X,Y,t);
title('Prob Z(x)>500ppm ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 1])
t=colorbar;
set(t,'fontsize',8);

p15=percent(fzc,seuils,ms,0.15);
figure(36);clf
p15=reshape(p15,nx,ny);
contourf(X,Y,p15);
title('15e percentile Pb ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 2500])
set(t,'fontsize',8);
t=colorbar;

p85=percent(fzc,seuils,ms,0.85);
figure(37);clf
p85=reshape(p85,nx,ny);
contourf(X,Y,p85);
title('85e percentile Pb ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 2500])
set(t,'fontsize',8);
t=colorbar;

figure(38);clf

contourf(X,Y,p85-p15);
title('Largeur 85e - 15e percentiles Pb ');
axis('equal')
set(gca,'fontsize',8);
t=colorbar;
caxis([0 2500])
set(t,'fontsize',8);
t=colorbar;